
/**Copffteer - a browser based game written in Coffeescript
    
    @author: Anton Erholt, <antonaut @ github>
    @copyright: © 2012 Anton Erholt, released under MIT licence:      

    Permission is hereby granted, free of
    charge, to any person obtaining a copy of this software and associated
    documentation files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy, modify,
    merge, publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so, subject to
    the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

/** constants
*/

(function() {
  var $, $h, AnimationComponent, Chopper, ClearComponent, Cloud, DEBUG, DIE, Entity, FPS, Game, GameplayState, HEIGHT, IntroState, MenuState, PCANV_HEIGHT, PCANV_WIDTH, State, Test, TextComponent, WIDTH, debug, dtime, g, isLittleEndian, keys, main, map, run, setupTools, show, __dbg,
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  WIDTH = 320;

  HEIGHT = 240;

  FPS = 60;

  PCANV_WIDTH = 160;

  PCANV_HEIGHT = 144;

  /** tools
  */

  show = function(x) {
    if ((typeof console !== "undefined" && console !== null) && (console.log != null)) {
      return console.log(x);
    } else {
      return alert(x);
    }
  };

  __dbg = [];

  debug = function(str) {
    return ($('#debug')).html(str);
  };

  map = function(fn, list) {
    var x, _i, _len, _results;
    _results = [];
    for (_i = 0, _len = list.length; _i < _len; _i++) {
      x = list[_i];
      _results.push(fn(x));
    }
    return _results;
  };

  setupTools = function() {
    var _ref;
    return (_ref = window.requestAnimationFrame) != null ? _ref : window.requestAnimationFrame = window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame || (function(callback, element) {
      return window.setTimeout(callback, 1000 / FPS);
    });
  };

  /** clearComponent
  */

  ClearComponent = (function() {

    function ClearComponent(px, py, width, height) {
      this.px = px;
      this.py = py;
      this.width = width;
      this.height = height;
    }

    ClearComponent.prototype.setNewDimension = function(width, height) {
      this.width = width;
      this.height = height;
    };

    ClearComponent.prototype.render = function(cxt, x, y) {
      cxt.clearRect(this.px, this.py, this.width, this.height);
      this.px = x;
      return this.py = y;
    };

    return ClearComponent;

  })();

  /** AnimationComponent
  */

  AnimationComponent = (function() {

    function AnimationComponent(sprites, delay) {
      this.sprites = sprites;
      this.delay = delay;
      this.current = 0;
      this.tick = 0;
    }

    AnimationComponent.prototype.setDelay = function(delay) {
      this.delay = delay;
    };

    AnimationComponent.prototype.render = function(cxt, x, y) {
      cxt.drawImage(this.sprites[this.current], x, y);
      if (this.tick % this.delay === 0) {
        this.current = (this.current + 1) % this.sprites.length;
      }
      return this.tick = this.tick + 1;
    };

    return AnimationComponent;

  })();

  /** textComponent
  */

  TextComponent = (function() {

    function TextComponent(cxt, font) {
      this.cxt = cxt;
      if (font == null) font = "9pt Courier";
      this.cxt.font = font;
      this.txts = [];
    }

    TextComponent.prototype.setFont = function(font) {
      return this.cxt.font = font;
    };

    TextComponent.prototype.typeWrite = function(text, xpos, ypos) {
      var a;
      a = {
        txt: text,
        x: xpos,
        y: ypos,
        cletter: 0,
        delay: 100,
        fadeTime: 1500,
        color: {
          r: 255,
          g: 255,
          b: 255
        },
        update: function() {
          if (this.cletter++ === this.txt.length) return this.delay--;
        },
        render: function(cxt) {}
      };
      return this.txts.push(a);
    };

    TextComponent.prototype.update = function(delta) {};

    TextComponent.prototype.render = function() {
      var t, _i, _len, _ref;
      this.cxt.clearRect(0, 0, WIDTH, HEIGHT);
      this.cxt.fillStyle = "rgb(" + this.i + "," + this.i + "," + this.i + ")";
      _ref = this.txts;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        t = _ref[_i];
        t.render(this.cxt);
      }
    };

    return TextComponent;

  })();

  /** entity
  */

  Entity = (function() {

    function Entity(id, xPos, yPos, maxspeed, width, height) {
      this.id = id;
      this.xPos = xPos;
      this.yPos = yPos;
      this.maxspeed = maxspeed;
      this.width = width;
      this.height = height;
      this.dx = 0;
      this.dy = 0;
      this.ddx = 0;
      this.ddy = 0;
      this.comps = {};
      this._roundXandY();
    }

    Entity.prototype._addComponent = function(name, component) {
      return this.comps[name] = component;
    };

    Entity.prototype._roundXandY = function() {
      this.x = Math.round(this.xPos);
      this.y = Math.round(this.yPos);
      if (this.xPos > 0) {
        this.xc = Math.min(Math.round(this.xPos, WIDTH));
      } else {
        this.xc = 0;
      }
      if (this.yPos > 0) {
        return this.yc = Math.min(Math.round(this.yPos, HEIGHT));
      } else {
        return this.yc = 0;
      }
    };

    Entity.prototype.update = function(delta) {
      this.dx = this.dx + this.ddx % this.maxspeed;
      this.dy = this.dy + this.ddy % this.maxspeed;
      this.xPos = this.xPos + this.dx;
      this.yPos = this.yPos + this.dy;
      return this._roundXandY();
    };

    Entity.prototype.intersects = function(entity) {
      if (this.yPos + this.height < entity.yPos || this.yPos > entity.yPos + entity.height || this.xPos + this.width < entity.xPos || this.xPos > entity.xPos + entity.width) {
        return false;
      } else {
        return true;
      }
    };

    return Entity;

  })();

  /** chopper
  */

  Chopper = (function(_super) {

    __extends(Chopper, _super);

    function Chopper(name, sprites) {
      Chopper.__super__.constructor.call(this, name, WIDTH / 2, HEIGHT / 2, 10, 80, 80);
      this._addComponent('sprites', new AnimationComponent(sprites, 7));
      this._addComponent('clear', new ClearComponent(this.x, this.y, this.width, this.height));
      this.dy = -0.8;
    }

    Chopper.prototype.toString = function() {
      return this.name;
    };

    Chopper.prototype.update = function(delta) {
      if (this.xPos > WIDTH) {
        this.xPos = 0;
      } else if (this.xPos < -this.width) {
        this.xPos = WIDTH;
      }
      if (this.yPos > HEIGHT) {
        this.yPos = 0;
      } else if (this.yPos < -this.height) {
        this.yPos = HEIGHT;
      }
      return Chopper.__super__.update.call(this, delta);
    };

    Chopper.prototype.clear = function(cxt) {
      return this.comps['clear'].render(cxt, this.x, this.y);
    };

    Chopper.prototype.render = function(cxt) {
      return this.comps['sprites'].render(cxt, this.x, this.y);
    };

    return Chopper;

  })(Entity);

  /** cloud
  */

  Cloud = (function(_super) {

    __extends(Cloud, _super);

    function Cloud(id, sprite) {
      var ranHeight, ranSpeed;
      this.sprite = sprite;
      ranHeight = Math.floor(Math.random() * HEIGHT - this.sprite.height);
      ranSpeed = 2 + Math.floor(Math.random() * 2);
      Cloud.__super__.constructor.call(this, id, -this.sprite.width, ranHeight, ranSpeed, this.sprite.width, this.sprite.height);
      this.dx = 1;
      this.dt = 0;
      this._addComponent('clear', new ClearComponent(this.x, this.y, this.width, this.height));
      this._roundXandY();
      this.resetPosition();
    }

    Cloud.prototype.resetPosition = function() {
      this.yPos = Math.floor(Math.random() * HEIGHT - this.sprite.height);
      this.maxspeed = 1 + Math.floor(Math.random() * 12);
      this.dx = this.maxspeed;
      return this.xPos = -this.width;
    };

    Cloud.prototype.update = function(delta) {
      Cloud.__super__.update.call(this, delta);
      this.dx = this.maxspeed * Math.abs(Math.sin(this.dt));
      if (this.xPos > WIDTH) this.resetPosition();
      return this.dt += 0.1 * Math.PI;
    };

    Cloud.prototype.clear = function(cxt) {
      return this.comps['clear'].render(cxt, this.x, this.y);
    };

    Cloud.prototype.render = function(cxt) {
      return cxt.drawImage(this.sprite, this.x, this.y);
    };

    return Cloud;

  })(Entity);

  /** state
  */

  State = (function() {

    function State(cxt, sprites) {
      this.cxt = cxt;
      this.sprites = sprites;
    }

    State.prototype.init = function() {};

    State.prototype.update = function() {};

    State.prototype.render = function() {};

    return State;

  })();

  /** gameplayState
  */

  GameplayState = (function(_super) {

    __extends(GameplayState, _super);

    function GameplayState(cxt, sprites) {
      var c, i, _len, _ref;
      GameplayState.__super__.constructor.call(this, cxt, sprites);
      this.cxt[1].fillStyle = '#44c';
      this.cxt[1].fillRect(0, 0, WIDTH, HEIGHT);
      this.chopper = new Chopper('player_one', this.sprites.chopper);
      this.entities = [];
      _ref = this.sprites.clouds;
      for (i = 0, _len = _ref.length; i < _len; i++) {
        c = _ref[i];
        this.entities.push(new Cloud(i, c));
      }
    }

    GameplayState.prototype.update = function(delta) {
      var delay, e, _i, _len, _ref, _results;
      this.chopper.update(delta);
      delay = 28;
      if (keys['Up']) {
        this.chopper.ddy = -0.01;
        delay -= 23;
      } else if (keys['Down']) {
        this.chopper.ddy = 0.01;
        delay += 1000;
      } else {
        this.chopper.ddy = 0.005;
      }
      if (keys['Left']) {
        this.chopper.ddx = -0.004;
        if (delay < 28) {
          delay -= 4;
        } else {
          delay -= 21;
        }
      } else if (keys['Right']) {
        this.chopper.ddx = 0.004;
        if (delay < 28) {
          delay -= 4;
        } else {
          delay -= 21;
        }
      } else {
        this.chopper.ddx = 0;
      }
      this.chopper.comps.sprites.setDelay(delay);
      _ref = this.entities;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        e = _ref[_i];
        _results.push(e.update());
      }
      return _results;
    };

    GameplayState.prototype.render = function() {
      var e, _i, _j, _len, _len2, _ref, _ref2;
      this.chopper.clear(this.cxt[2]);
      _ref = this.entities;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        e = _ref[_i];
        e.clear(this.cxt[3]);
      }
      this.chopper.render(this.cxt[2]);
      _ref2 = this.entities;
      for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
        e = _ref2[_j];
        e.render(this.cxt[3]);
      }
    };

    return GameplayState;

  })(State);

  /** menuState
  */

  MenuState = (function(_super) {

    __extends(MenuState, _super);

    function MenuState(cxt, sprites) {
      this.cxt = cxt;
      this.sprites = sprites;
      MenuState.__super__.constructor.call(this);
    }

    return MenuState;

  })(State);

  /** introState
  */

  IntroState = (function(_super) {

    __extends(IntroState, _super);

    function IntroState(cxt, sprites) {
      this.cxt = cxt;
      this.sprites = sprites;
      this.cxt[0].fillStyle = "#000";
      this.cxt[0].fillRect(0, 0, WIDTH, HEIGHT);
      this.tc = new TextComponent(this.cxt[1]);
    }

    IntroState.prototype.playIntro = function() {
      var dy, line, p, _i, _len, _results;
      p = ['Ctrl-Alt-Delete Studios', 'proudly presents', 'COPFFTEER', 'the super awesome chopper game'];
      this.tc.setFont("7pt PressStart2P");
      dy = 25;
      _results = [];
      for (_i = 0, _len = p.length; _i < _len; _i++) {
        line = p[_i];
        _results.push(this.tc.typeWrite(line, WIDTH / 2 - ((this.cxt[1].measureText(line)).width / 2), HEIGHT / 2 - 50 + dy));
      }
      return _results;
    };

    IntroState.prototype.update = function(delta) {};

    IntroState.prototype.render = function() {
      return this.playIntro();
    };

    return IntroState;

  })(State);

  /** game
  */

  /** Class which holds the critical components of the game.
  */

  Game = (function() {

    function Game() {
      var bg, bgCxt, initCanvas, initSprites, k, l1, l1Cxt, l2, l2Cxt, l3, l3Cxt, v, _ref, _ref2, _ref3, _ref4, _ref5;
      this.sprites = {};
      initSprites = function(range, path, suffix) {
        var i, img, obj, _i, _len;
        obj = [];
        for (_i = 0, _len = range.length; _i < _len; _i++) {
          i = range[_i];
          img = new Image;
          img.src = path + ("" + i) + suffix;
          obj.push(img);
        }
        return obj;
      };
      this.sprites.chopper = initSprites([2, 3, 4, 5, 6, 7], "images/copter/copter", ".png");
      this.sprites.clouds = initSprites([1, 2, 3, 4], "images/clouds/cloud", ".png");
      this.sprites.smokepuffs = initSprites([1, 2, 3, 4, 5], "images/smokepuffs/smokepuff", ".png");
      initCanvas = function(id) {
        var canvas, cxt;
        canvas = ($(id))[0];
        canvas.height = HEIGHT;
        canvas.width = WIDTH;
        cxt = canvas.getContext('2d');
        return [canvas, cxt];
      };
      _ref = initCanvas('#la0'), bg = _ref[0], bgCxt = _ref[1];
      _ref2 = initCanvas('#la1'), l1 = _ref2[0], l1Cxt = _ref2[1];
      _ref3 = initCanvas('#la2'), l2 = _ref3[0], l2Cxt = _ref3[1];
      _ref4 = initCanvas('#la3'), l3 = _ref4[0], l3Cxt = _ref4[1];
      this.cxt = [bgCxt, l1Cxt, l2Cxt, l3Cxt];
      this.can = [bg, l1, l2, l3];
      this.states = {
        'menu': new MenuState(this.cxt, this.sprites),
        'intro': new IntroState(this.cxt, this.sprites),
        'gameplay': new GameplayState(this.cxt, this.sprites)
      };
      _ref5 = this.states;
      for (k in _ref5) {
        v = _ref5[k];
        v.init();
      }
      this.currentState = this.states['intro'];
    }

    Game.prototype.update = function(delta) {
      return this.currentState.update();
    };

    Game.prototype.draw = function() {
      return this.currentState.render();
    };

    Game.prototype.switchState = function(stateName) {
      return this.currentState = this.states[stateName];
    };

    return Game;

  })();

  DIE = function() {
    return alert("YOU DIED!");
  };

  /** main
  */

  /*
    VARIABLES
  */

  DEBUG = true;

  $ = jQuery;

  $h = function(x) {
    return ($(x))[0];
  };

  g = null;

  dtime = 0;

  keys = {
    'Left': false,
    'Up': false,
    'Right': false,
    'Down': false,
    'w': false,
    's': false,
    'a': false,
    'd': false
  };

  isLittleEndian = true;

  /**
    main
    exports to window.main
    binds events and such
  */

  main = function() {
    var $g, el,
      _this = this;
    setupTools();
    ($('body')).append("<div id=\"debug\" style=\"font-family: PressStart2P\"></div>");
    el = ($('html'))[0];
    /* Bind events
    */
    ($('#fullscreen'))[0].addEventListener('click', function() {
      var rfs, wscript;
      rfs = el.requestFullScreen || el.webkitRequestFullScreen || el.mozRequestFullScreen || el.msRequestFullScreen;
      if ((rfs != null) || typeof rfs !== 'undefined') {
        return rfs.call(el);
      } else if (typeof window.ActiveXObject !== 'undefined') {
        wscript = new ActiveXObject("WScript.shell");
        if (wscript != null) return wscript.SendKeys('{F11}');
      }
    });
    ($('#game')).width(WIDTH);
    ($('#game')).height(HEIGHT);
    ($(document)).keydown(function(evt) {
      debug("<p>keycode: " + evt.which + "</p>");
      switch (evt.which) {
        case 13:
          return evt.preventDefault();
        case 37:
          return keys['Left'] = true;
        case 38:
          return keys['Up'] = true;
        case 39:
          return keys['Right'] = true;
        case 40:
          return keys['Down'] = true;
        case 65:
          return keys['a'] = true;
        case 87:
          return keys['w'] = true;
        case 68:
          return keys['d'] = true;
        case 83:
          return keys['s'] = true;
      }
    });
    ($(document)).keyup(function(evt) {
      switch (evt.which) {
        case 13:
          return evt.preventDefault();
        case 37:
          return keys['Left'] = false;
        case 38:
          return keys['Up'] = false;
        case 39:
          return keys['Right'] = false;
        case 40:
          return keys['Down'] = false;
        case 65:
          return keys['a'] = false;
        case 87:
          return keys['w'] = false;
        case 68:
          return keys['d'] = false;
        case 83:
          return keys['s'] = false;
      }
    });
    g = new Game;
    $g = $('#game');
    $g.children().css('height:100%;width:100%;');
    return run();
  };

  /** 
    run
    Should probably be called 'mainloop'.
  */

  run = function() {
    var delta;
    delta = ((new Date).getTime() - dtime) * 1e-3;
    dtime = (new Date).getTime();
    g.update(delta);
    g.draw();
    return window.requestAnimationFrame(run);
  };

  window.main = main;

  /** test
    @author Anton Erholt
  */

  Test = (function() {

    function Test() {}

    Test.constructor = function() {
      return testEndianess();
    };

    Test.testEndianess = function() {
      var a, b, c;
      a = new ArrayBuffer(16);
      b = new UInt32Array(a);
      c = new Uint8ClampedArray(a);
      b1[0] = 0;
      if (c[0] === 0x0a && c[1] === 0x0b && c[2] === 0x0c && c[3] === 0x0d) {
        return isLittleEndian = false;
      }
    };

    return Test;

  })();

}).call(this);
